@@ -1,24 +1,22 @@
+/* SPDX-License-Identifier: GPL-2.0-or-later */
+
 /*
  * Copyright (c) 2020, Ampere Computing LLC.
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
  */
 
-
 #ifndef __NON_STANDARD_AMPERE_H
 #define __NON_STANDARD_AMPERE_H
 
-#include "ras-events.h"
 #include <traceevent/event-parse.h>
 
-#define SOCKET_NUM(x) ((x >> 14) & 0x3)
-#define PAYLOAD_TYPE(x) ((x >> 6) & 0x3)
-#define TYPE(x) (x & 0x3f)
-#define INSTANCE(x) (x & 0x3fff)
+#include "types.h"
+
+struct ras_ns_ev_decoder;
+
+#define SOCKET_NUM(x) (((x) >> 14) & 0x3)
+#define PAYLOAD_TYPE(x) (((x) >> 6) & 0x3)
+#define TYPE(x) ((x) & 0x3f)
+#define INSTANCE(x) ((x) & 0x3fff)
 #define AMP_PAYLOAD0_BUF_LEN   1024
 #define PAYLOAD_TYPE_0         0x00
 #define PAYLOAD_TYPE_1         0x01
