@@ -133,6 +133,7 @@ int ras_cxl_poison_event_handler(struct trace_seq *s,
 	struct ras_events *ras = context;
 	struct ras_cxl_poison_event ev;
 
+	trace_seq_printf(s, "%s ", loglevel_str[LOGLEVEL_ERR]);
 	get_timestamp(s, record, ras, (char *)&ev.timestamp, sizeof(ev.timestamp));
 	if (trace_seq_printf(s, "%s ", ev.timestamp) <= 0)
 		return -1;
@@ -192,7 +193,13 @@ int ras_cxl_poison_event_handler(struct trace_seq *s,
 	if (tep_get_field_val(s, event, "hpa", record, &val, 1) < 0)
 		return -1;
 	ev.hpa = val;
-	if (trace_seq_printf(s, "poison list: hpa:0x%llx ", (unsigned long long)ev.hpa) <= 0)
+	if (trace_seq_printf(s, "hpa:0x%llx ", (unsigned long long)ev.hpa) <= 0)
+		return -1;
+
+	if (tep_get_field_val(s, event, "hpa_alias0", record, &val, 1) < 0)
+		return -1;
+	ev.hpa_alias0 = val;
+	if (trace_seq_printf(s, "hpa_alias0:0x%llx ", (unsigned long long)ev.hpa_alias0) <= 0)
 		return -1;
 
 	if (tep_get_field_val(s, event, "dpa", record, &val, 1) < 0)
@@ -345,6 +352,7 @@ int ras_cxl_aer_ue_event_handler(struct trace_seq *s,
 	struct ras_cxl_aer_ue_event ev;
 
 	memset(&ev, 0, sizeof(ev));
+	trace_seq_printf(s, "%s ", loglevel_str[LOGLEVEL_CRIT]);
 	get_timestamp(s, record, ras, (char *)&ev.timestamp, sizeof(ev.timestamp));
 	if (trace_seq_printf(s, "%s ", ev.timestamp) <= 0)
 		return -1;
@@ -431,6 +439,7 @@ int ras_cxl_aer_ce_event_handler(struct trace_seq *s,
 	struct ras_events *ras = context;
 	struct ras_cxl_aer_ce_event ev;
 
+	trace_seq_printf(s, "%s ", loglevel_str[LOGLEVEL_ERR]);
 	get_timestamp(s, record, ras, (char *)&ev.timestamp, sizeof(ev.timestamp));
 	if (trace_seq_printf(s, "%s ", ev.timestamp) <= 0)
 		return -1;
@@ -516,6 +525,7 @@ int ras_cxl_overflow_event_handler(struct trace_seq *s,
 	struct ras_cxl_overflow_event ev;
 
 	memset(&ev, 0, sizeof(ev));
+	trace_seq_printf(s, "%s ", loglevel_str[LOGLEVEL_ERR]);
 	get_timestamp(s, record, ras, (char *)&ev.timestamp, sizeof(ev.timestamp));
 	if (trace_seq_printf(s, "%s ", ev.timestamp) <= 0)
 		return -1;
@@ -624,6 +634,8 @@ static int ras_cxl_print_component_id(struct trace_seq *s, uint8_t *comp_id,
 #define CXL_EVENT_RECORD_FLAG_PERF_DEGRADED	BIT(4)
 #define CXL_EVENT_RECORD_FLAG_HW_REPLACE	BIT(5)
 #define CXL_EVENT_RECORD_FLAG_MAINT_OP_SUB_CLASS_VALID	BIT(6)
+#define CXL_EVENT_RECORD_FLAG_LD_ID_VALID	BIT(7)
+#define CXL_EVENT_RECORD_FLAG_HEAD_ID_VALID	BIT(8)
 
 static const struct  cxl_event_flags cxl_hdr_flags[] = {
 	{ .bit = CXL_EVENT_RECORD_FLAG_PERMANENT, .flag = "PERMANENT_CONDITION" },
@@ -631,6 +643,8 @@ static const struct  cxl_event_flags cxl_hdr_flags[] = {
 	{ .bit = CXL_EVENT_RECORD_FLAG_PERF_DEGRADED, .flag = "PERFORMANCE_DEGRADED" },
 	{ .bit = CXL_EVENT_RECORD_FLAG_HW_REPLACE, .flag = "HARDWARE_REPLACEMENT_NEEDED" },
 	{ .bit = CXL_EVENT_RECORD_FLAG_MAINT_OP_SUB_CLASS_VALID, .flag = "MAINT_OP_SUB_CLASS_VALID" },
+	{ .bit = CXL_EVENT_RECORD_FLAG_LD_ID_VALID, .flag = "LOGICAL_DEV_ID_VALID" },
+	{ .bit = CXL_EVENT_RECORD_FLAG_HEAD_ID_VALID, .flag = "DEV_HEAD_ID_VALID" },
 };
 
 static int handle_ras_cxl_common_hdr(struct trace_seq *s,
@@ -714,11 +728,29 @@ static int handle_ras_cxl_common_hdr(struct trace_seq *s,
 	if (trace_seq_printf(s, "hdr_maint_op_class:%u ", hdr->hdr_maint_op_class) <= 0)
 		return -1;
 
-	if (tep_get_field_val(s,  event, "hdr_maint_op_sub_class", record, &val, 1) < 0)
-		return -1;
-	hdr->hdr_maint_op_sub_class = val;
-	if (trace_seq_printf(s, "hdr_maint_op_sub_class:%u ", hdr->hdr_maint_op_sub_class) <= 0)
-		return -1;
+	if (hdr->hdr_flags & CXL_EVENT_RECORD_FLAG_MAINT_OP_SUB_CLASS_VALID) {
+		if (tep_get_field_val(s,  event, "hdr_maint_op_sub_class", record, &val, 1) < 0)
+			return -1;
+		hdr->hdr_maint_op_sub_class = val;
+		if (trace_seq_printf(s, "hdr_maint_op_sub_class:%u ", hdr->hdr_maint_op_sub_class) <= 0)
+			return -1;
+	}
+
+	if (hdr->hdr_flags & CXL_EVENT_RECORD_FLAG_LD_ID_VALID) {
+		if (tep_get_field_val(s,  event, "hdr_ld_id", record, &val, 1) < 0)
+			return -1;
+		hdr->hdr_ld_id = val;
+		if (trace_seq_printf(s, "hdr_ld_id:0x%x ", hdr->hdr_ld_id) <= 0)
+			return -1;
+	}
+
+	if (hdr->hdr_flags & CXL_EVENT_RECORD_FLAG_HEAD_ID_VALID) {
+		if (tep_get_field_val(s,  event, "hdr_head_id", record, &val, 1) < 0)
+			return -1;
+		hdr->hdr_head_id = val;
+		if (trace_seq_printf(s, "hdr_head_id:0x%x ", hdr->hdr_head_id) <= 0)
+			return -1;
+	}
 
 	return 0;
 }
@@ -733,6 +765,7 @@ int ras_cxl_generic_event_handler(struct trace_seq *s,
 	const uint8_t *buf;
 
 	memset(&ev, 0, sizeof(ev));
+	trace_seq_printf(s, "%s ", loglevel_str[LOGLEVEL_ERR]);
 	if (handle_ras_cxl_common_hdr(s, record, event, context, &ev.hdr) < 0)
 		return -1;
 
@@ -848,6 +881,7 @@ int ras_cxl_general_media_event_handler(struct trace_seq *s,
 	struct ras_cxl_general_media_event ev;
 
 	memset(&ev, 0, sizeof(ev));
+	trace_seq_printf(s, "%s ", loglevel_str[LOGLEVEL_ERR]);
 	if (handle_ras_cxl_common_hdr(s, record, event, context, &ev.hdr) < 0)
 		return -1;
 
@@ -906,6 +940,12 @@ int ras_cxl_general_media_event_handler(struct trace_seq *s,
 	if (trace_seq_printf(s, "hpa:0x%llx ", (unsigned long long)ev.hpa) <= 0)
 		return -1;
 
+	if (tep_get_field_val(s, event, "hpa_alias0", record, &val, 1) < 0)
+		return -1;
+	ev.hpa_alias0 = val;
+	if (trace_seq_printf(s, "hpa_alias0:0x%llx ", (unsigned long long)ev.hpa_alias0) <= 0)
+		return -1;
+
 	ev.region = tep_get_field_raw(s, event, "region_name", record, &len, 1);
 	if (!ev.region)
 		return -1;
@@ -972,21 +1012,23 @@ int ras_cxl_general_media_event_handler(struct trace_seq *s,
 		}
 	}
 
-	if (tep_get_field_val(s,  event, "cme_threshold_ev_flags", record, &val, 1) < 0)
-		return -1;
-	ev.cme_threshold_ev_flags = val;
-	if (trace_seq_printf(s, "Advanced Programmable CME threshold Event Flags:") <= 0)
-		return -1;
-	if (decode_cxl_event_flags(s, ev.cme_threshold_ev_flags,
-				   cxl_cme_threshold_ev_flags,
-				   ARRAY_SIZE(cxl_cme_threshold_ev_flags)) < 0)
-		return -1;
+	if (ev.descriptor & CXL_GMER_EVT_DESC_THRESHOLD_EVENT) {
+		if (tep_get_field_val(s,  event, "cme_threshold_ev_flags", record, &val, 1) < 0)
+			return -1;
+		ev.cme_threshold_ev_flags = val;
+		if (trace_seq_printf(s, "Advanced Programmable CME threshold Event Flags:") <= 0)
+			return -1;
+		if (decode_cxl_event_flags(s, ev.cme_threshold_ev_flags,
+					   cxl_cme_threshold_ev_flags,
+					   ARRAY_SIZE(cxl_cme_threshold_ev_flags)) < 0)
+			return -1;
 
-	if (tep_get_field_val(s,  event, "cme_count", record, &val, 1) < 0)
-		return -1;
-	ev.cme_count = val;
-	if (trace_seq_printf(s, "Corrected Memory Error Count:%u ", ev.cme_count) <= 0)
-		return -1;
+		if (tep_get_field_val(s,  event, "cme_count", record, &val, 1) < 0)
+			return -1;
+		ev.cme_count = val;
+		if (trace_seq_printf(s, "Corrected Memory Error Count:%u ", ev.cme_count) <= 0)
+			return -1;
+	}
 
 	/* Insert data into the SGBD */
 #ifdef HAVE_SQLITE3
@@ -1038,6 +1080,7 @@ int ras_cxl_dram_event_handler(struct trace_seq *s,
 	struct ras_cxl_dram_event ev;
 
 	memset(&ev, 0, sizeof(ev));
+	trace_seq_printf(s, "%s ", loglevel_str[LOGLEVEL_ERR]);
 	if (handle_ras_cxl_common_hdr(s, record, event, context, &ev.hdr) < 0)
 		return -1;
 
@@ -1047,12 +1090,6 @@ int ras_cxl_dram_event_handler(struct trace_seq *s,
 	if (trace_seq_printf(s, "dpa:0x%llx ", (unsigned long long)ev.dpa) <= 0)
 		return -1;
 
-	if (tep_get_field_val(s, event, "hpa", record, &val, 1) < 0)
-		return -1;
-	ev.hpa = val;
-	if (trace_seq_printf(s, "hpa:0x%llx ", (unsigned long long)ev.hpa) <= 0)
-		return -1;
-
 	if (tep_get_field_val(s,  event, "dpa_flags", record, &val, 1) < 0)
 		return -1;
 	ev.dpa_flags = val;
@@ -1104,6 +1141,12 @@ int ras_cxl_dram_event_handler(struct trace_seq *s,
 	if (trace_seq_printf(s, "hpa:0x%llx ", (unsigned long long)ev.hpa) <= 0)
 		return -1;
 
+	if (tep_get_field_val(s, event, "hpa_alias0", record, &val, 1) < 0)
+		return -1;
+	ev.hpa_alias0 = val;
+	if (trace_seq_printf(s, "hpa_alias0:0x%llx ", (unsigned long long)ev.hpa_alias0) <= 0)
+		return -1;
+
 	ev.region = tep_get_field_raw(s, event, "region_name", record, &len, 1);
 	if (!ev.region)
 		return -1;
@@ -1199,7 +1242,7 @@ int ras_cxl_dram_event_handler(struct trace_seq *s,
 	}
 
 #ifdef HAVE_MEMORY_CE_PFA
-	/* Page offline for CE when threeshold is set */
+	/* Page offline for CE when threshold is set */
 	if (!(ev.descriptor & CXL_GMER_EVT_DESC_UNCORRECTABLE_EVENT) &&
 	    (ev.descriptor & CXL_GMER_EVT_DESC_THRESHOLD_EVENT))
 		ras_hw_threshold_pageoffline(ev.hpa);
@@ -1229,21 +1272,23 @@ int ras_cxl_dram_event_handler(struct trace_seq *s,
 		}
 	}
 
-	if (tep_get_field_val(s,  event, "cme_threshold_ev_flags", record, &val, 1) < 0)
-		return -1;
-	ev.cme_threshold_ev_flags = val;
-	if (trace_seq_printf(s, "Advanced Programmable CME threshold Event Flags:") <= 0)
-		return -1;
-	if (decode_cxl_event_flags(s, ev.cme_threshold_ev_flags,
-				   cxl_cme_threshold_ev_flags,
-				   ARRAY_SIZE(cxl_cme_threshold_ev_flags)) < 0)
-		return -1;
+	if (ev.descriptor & CXL_GMER_EVT_DESC_THRESHOLD_EVENT) {
+		if (tep_get_field_val(s,  event, "cme_threshold_ev_flags", record, &val, 1) < 0)
+			return -1;
+		ev.cme_threshold_ev_flags = val;
+		if (trace_seq_printf(s, "Advanced Programmable CME threshold Event Flags:") <= 0)
+			return -1;
+		if (decode_cxl_event_flags(s, ev.cme_threshold_ev_flags,
+					   cxl_cme_threshold_ev_flags,
+					   ARRAY_SIZE(cxl_cme_threshold_ev_flags)) < 0)
+			return -1;
 
-	if (tep_get_field_val(s,  event, "cvme_count", record, &val, 1) < 0)
-		return -1;
-	ev.cvme_count = val;
-	if (trace_seq_printf(s, "CVME Count:%u ", ev.cvme_count) <= 0)
-		return -1;
+		if (tep_get_field_val(s,  event, "cvme_count", record, &val, 1) < 0)
+			return -1;
+		ev.cvme_count = val;
+		if (trace_seq_printf(s, "CVME Count:%u ", ev.cvme_count) <= 0)
+			return -1;
+	}
 
 	/* Insert data into the SGBD */
 #ifdef HAVE_SQLITE3
@@ -1473,3 +1518,157 @@ int ras_cxl_memory_module_event_handler(struct trace_seq *s,
 
 	return 0;
 }
+
+/*
+ * Memory Sparing Event Record - MSER
+ *
+ * CXL rev 3.2 section 8.2.10.2.1.4; Table 8-60
+ */
+#define CXL_MSER_VALID_CHANNEL			BIT(0)
+#define CXL_MSER_VALID_RANK			BIT(1)
+#define CXL_MSER_VALID_NIBBLE			BIT(2)
+#define CXL_MSER_VALID_BANK_GROUP		BIT(3)
+#define CXL_MSER_VALID_BANK			BIT(4)
+#define CXL_MSER_VALID_ROW			BIT(5)
+#define CXL_MSER_VALID_COLUMN			BIT(6)
+#define CXL_MSER_VALID_COMPONENT_ID		BIT(7)
+#define CXL_MSER_VALID_COMPONENT_ID_FORMAT	BIT(8)
+#define CXL_MSER_VALID_SUB_CHANNEL		BIT(9)
+
+#define CXL_MSER_QUERY_RES_FLAG		BIT(0)
+#define CXL_MSER_HARD_SPARING_FLAG	BIT(1)
+#define CXL_MSER_DEV_INITIATED_FLAG	BIT(2)
+
+static const struct cxl_event_flags cxl_mser_flags[] = {
+	{ .bit = CXL_MSER_QUERY_RES_FLAG, .flag = "QUERY_RESOURCES" },
+	{ .bit = CXL_MSER_HARD_SPARING_FLAG, .flag = "HARD_SPARING" },
+	{ .bit = CXL_MSER_DEV_INITIATED_FLAG, .flag = "DEVICE_INITIATED" },
+};
+
+int ras_cxl_memory_sparing_event_handler(struct trace_seq *s,
+					 struct tep_record *record,
+					 struct tep_event *event, void *context)
+{
+	int len, i, rc;
+	unsigned long long val;
+	struct ras_cxl_memory_sparing_event ev;
+
+	memset(&ev, 0, sizeof(ev));
+	if (handle_ras_cxl_common_hdr(s, record, event, context, &ev.hdr) < 0)
+		return -1;
+
+	if (tep_get_field_val(s, event, "flags", record, &val, 1) < 0)
+		return -1;
+	ev.flags = val;
+	if (trace_seq_printf(s, "flags:0x%x ", ev.flags) <= 0)
+		return -1;
+	if (decode_cxl_event_flags(s, ev.flags, cxl_mser_flags,
+				   ARRAY_SIZE(cxl_mser_flags)) < 0)
+		return -1;
+
+	if (tep_get_field_val(s,  event, "result", record, &val, 1) < 0)
+		return -1;
+	ev.result = val;
+	if (trace_seq_printf(s, "result:0x%x ", ev.result) <= 0)
+		return -1;
+
+	if (tep_get_field_val(s,  event, "validity_flags", record, &val, 1) < 0)
+		return -1;
+	ev.validity_flags = val;
+
+	if (tep_get_field_val(s,  event, "res_avail", record, &val, 1) < 0)
+		return -1;
+	ev.res_avail = val;
+	if (trace_seq_printf(s, "spare resources available:%u ", ev.res_avail) <= 0)
+		return -1;
+
+	if (ev.validity_flags & CXL_MSER_VALID_CHANNEL) {
+		if (tep_get_field_val(s,  event, "channel", record, &val, 1) < 0)
+			return -1;
+		ev.channel = val;
+		if (trace_seq_printf(s, "channel:%u ", ev.channel) <= 0)
+			return -1;
+	}
+
+	if (ev.validity_flags & CXL_MSER_VALID_SUB_CHANNEL) {
+		if (tep_get_field_val(s,  event, "sub_channel", record, &val, 1) < 0)
+			return -1;
+		ev.sub_channel = val;
+		if (trace_seq_printf(s, "sub_channel:%u ", ev.sub_channel) <= 0)
+			return -1;
+	}
+
+	if (ev.validity_flags & CXL_MSER_VALID_RANK) {
+		if (tep_get_field_val(s,  event, "rank", record, &val, 1) < 0)
+			return -1;
+		ev.rank = val;
+		if (trace_seq_printf(s, "rank:%u ", ev.rank) <= 0)
+			return -1;
+	}
+
+	if (ev.validity_flags & CXL_MSER_VALID_NIBBLE) {
+		if (tep_get_field_val(s,  event, "nibble_mask", record, &val, 1) < 0)
+			return -1;
+		ev.nibble_mask = val;
+		if (trace_seq_printf(s, "nibble_mask:%u ", ev.nibble_mask) <= 0)
+			return -1;
+	}
+
+	if (ev.validity_flags & CXL_MSER_VALID_BANK_GROUP) {
+		if (tep_get_field_val(s,  event, "bank_group", record, &val, 1) < 0)
+			return -1;
+		ev.bank_group = val;
+		if (trace_seq_printf(s, "bank_group:%u ", ev.bank_group) <= 0)
+			return -1;
+	}
+
+	if (ev.validity_flags & CXL_MSER_VALID_BANK) {
+		if (tep_get_field_val(s,  event, "bank", record, &val, 1) < 0)
+			return -1;
+		ev.bank = val;
+		if (trace_seq_printf(s, "bank:%u ", ev.bank) <= 0)
+			return -1;
+	}
+
+	if (ev.validity_flags & CXL_MSER_VALID_ROW) {
+		if (tep_get_field_val(s,  event, "row", record, &val, 1) < 0)
+			return -1;
+		ev.row = val;
+		if (trace_seq_printf(s, "row:%u ", ev.row) <= 0)
+			return -1;
+	}
+
+	if (ev.validity_flags & CXL_MSER_VALID_COLUMN) {
+		if (tep_get_field_val(s,  event, "column", record, &val, 1) < 0)
+			return -1;
+		ev.column = val;
+		if (trace_seq_printf(s, "column:%u ", ev.column) <= 0)
+			return -1;
+	}
+
+	if (ev.validity_flags & CXL_MSER_VALID_COMPONENT_ID) {
+		ev.comp_id = tep_get_field_raw(s, event, "comp_id", record, &len, 1);
+		if (!ev.comp_id)
+			return -1;
+		if (trace_seq_printf(s, "comp_id:") <= 0)
+			return -1;
+		for (i = 0; i < CXL_EVENT_GEN_MED_COMP_ID_SIZE; i++) {
+			if (trace_seq_printf(s, "%02x ", ev.comp_id[i]) <= 0)
+				break;
+		}
+
+		if (ev.validity_flags & CXL_MSER_VALID_COMPONENT_ID_FORMAT) {
+			if (trace_seq_printf(s, "comp_id_pldm_valid_flags:") <= 0)
+				return -1;
+			if (decode_cxl_event_flags(s, ev.comp_id[0], cxl_pldm_comp_id_flags,
+						   ARRAY_SIZE(cxl_pldm_comp_id_flags)) < 0)
+				return -1;
+
+			rc = ras_cxl_print_component_id(s, ev.comp_id, ev.entity_id, ev.res_id);
+			if (rc)
+				return rc;
+		}
+	}
+
+	return 0;
+}
