@@ -228,7 +228,7 @@ ret:
 	return ret;
 }
 
-int register_mce_handler(struct ras_events *ras, unsigned int ncpus)
+int init_mce_priv(struct ras_events *ras)
 {
 	int rc;
 	struct mce_priv *mce;
@@ -249,6 +249,12 @@ int register_mce_handler(struct ras_events *ras, unsigned int ncpus)
 		ras->mce_priv = NULL;
 		return rc;
 	}
+
+	return rc;
+}
+
+static void set_imc_log(struct mce_priv *mce, unsigned int ncpus)
+{
 	switch (mce->cputype) {
 	case CPU_SANDY_BRIDGE_EP:
 	case CPU_IVY_BRIDGE_EPEX:
@@ -259,6 +265,17 @@ int register_mce_handler(struct ras_events *ras, unsigned int ncpus)
 	default:
 		break;
 	}
+}
+
+int register_mce_handler(struct ras_events *ras, unsigned int ncpus)
+{
+	int rc;
+
+	rc = init_mce_priv(ras);
+	if (rc)
+		return rc;
+
+	set_imc_log(ras->mce_priv, ncpus);
 
 	return rc;
 }
@@ -267,14 +284,22 @@ int register_mce_handler(struct ras_events *ras, unsigned int ncpus)
  * End of mcelog's code
  */
 
-static void report_mce_event(struct ras_events *ras,
-			     struct tep_record *record,
-			     struct trace_seq *s, struct mce_event *e)
+void report_mce_event(struct ras_events *ras, struct tep_record *record,
+		      struct trace_seq *s, struct mce_event *e)
 {
 	time_t now;
 	struct tm *tm;
 	struct mce_priv *mce = ras->mce_priv;
+	const char *level;
 
+	if (e->status & MCI_STATUS_UC)
+		level = loglevel_str[LOGLEVEL_CRIT];
+	else if (e->status & MCI_STATUS_DEFERRED)
+		level = loglevel_str[LOGLEVEL_CRIT];
+	else
+		level = loglevel_str[LOGLEVEL_ERR];
+
+	trace_seq_printf(s, "%s ", level);
 	/*
 	 * Newer kernels (3.10-rc1 or upper) provide an uptime clock.
 	 * On previous kernels, the way to properly generate an event would
@@ -284,10 +309,14 @@ static void report_mce_event(struct ras_events *ras,
 	 * not available (legacy kernels).
 	 */
 
-	if (ras->use_uptime)
-		now = record->ts / user_hz + ras->uptime_diff;
-	else
-		now = time(NULL);
+	if (!e->erst) {
+		if (ras->use_uptime)
+			now = record->ts / user_hz + ras->uptime_diff;
+		else
+			now = time(NULL);
+	} else {
+		now = e->walltime;
+	}
 
 	tm = localtime(&now);
 	if (tm)
